/**
 *  
 *  @header AgoraChatConversation.h
 *  @abstract Chat conversation
 *  @author Hyphenate
 *  @version 3.00
 */

#import <Foundation/Foundation.h>

#import "AgoraChatMessageBody.h"

/**
 *  
 *  The conversation type.
 */
typedef NS_ENUM(NSInteger, AgoraChatConversationType) {
    AgoraChatConversationTypeChat = 0,    /**  The one-to-one chat type. */
    AgoraChatConversationTypeGroupChat,    /**  The group chat type.*/
    AgoraChatConversationTypeChatRoom,      /**  The chat room type.*/
};

/**
 *  
 *  The message search direction type.
 */
typedef NS_ENUM(NSInteger, AgoraChatMessageSearchDirection) {
    AgoraChatMessageSearchDirectionUp  = 0,    /**  The search older messages type.*/
    AgoraChatMessageSearchDirectionDown        /**  The search newer messages type.*/
};

@class AgoraChatMessage;
@class AgoraChatError;

/**
 *  
 *  The chat conversation.
 */
@interface AgoraChatConversation : NSObject

/**
 *  
 *  The conversation ID.
 *  For one-to-one chat，conversation ID is to chat user's name.
 *  For group chat, conversation ID is groupID(), different with getGroupName().
 *  For chat room, conversation ID is chatroom ID, different with chat room name().
 *  For help desk, it is same with one-to-one chat, conversation ID is also chat user's name.
 */
@property (nonatomic, copy, readonly) NSString *conversationId;

/**
 *  
 *  The conversation type.
 */
@property (nonatomic, assign, readonly) AgoraChatConversationType type;

/**
 *  
 *  The number of unread messages.
 */
@property (nonatomic, assign, readonly) int unreadMessagesCount;

/**
 *  
 *  Message count
 */
@property (nonatomic, assign, readonly) int messagesCount;

/**
 *  
 *  The conversation extension property.
 */
@property (nonatomic, copy) NSDictionary *ext;

/*!
 *  
 *  The latest message in the conversation.
 */
@property (nonatomic, assign) BOOL isChatThread;

/*!
 *  
 *  The latest message in the conversation.
 */
@property (nonatomic, strong, readonly) AgoraChatMessage *latestMessage;

/**
 *  
 *  Gets the last received message.
 *
 *  @result The message instance.
 */
- (AgoraChatMessage * _Nullable)lastReceivedMessage;

/**
 *  
 *  Inserts a message to a conversation in local database and SDK will update the last message automatically.
 * 
 *  The conversation ID of the message should be the same as conversation ID of the conversation in order to insert the message into the conversation correctly. The inserting message will be inserted based on timestamp.
 * 
 *  Method AgoraChatConversation insertMessage:error: = AgoraChatManager importMsessage:completion: + update conversation latest message
 *
 *  @param aMessage The message instance.
 *  @param pError   The error information if the method fails: Error.
 */
- (void)insertMessage:(AgoraChatMessage *_Nonnull)aMessage
                error:(AgoraChatError ** _Nullable)pError;

/**
 *  
 *  Inserts a message to the end of a conversation in local database. The conversationId of the message should be the same as the conversationId of the conversation in order to insert the message into the conversation correctly.
 *
 *  @param aMessage The message instance.
 *  @param pError   The error information if the method fails: Error.
 *
 */
- (void)appendMessage:(AgoraChatMessage *_Nonnull)aMessage
                error:(AgoraChatError **_Nullable)pError;

/**
 *  
 *  Deletes a message.
 *
 *  @param aMessageId   The ID of the message to be deleted.
 *  @param pError       The error information if the method fails: Error.
 *
 */
- (void)deleteMessageWithId:(NSString *_Nonnull)aMessageId
                      error:(AgoraChatError ** _Nullable)pError;

/**
 *  
 *  Deletes all messages of the conversation from memory cache and local database.
 *
 *  @param pError       The error information if the method fails: Error.
 */
- (void)deleteAllMessages:(AgoraChatError ** _Nullable)pError;

/**
 *  
 *  Uses this method to update a message in local database. Changing properties will affect data in database.
 * 
 *  The latestMessage of the conversation and other properties will be updated accordingly. The messageID of the message cannot be updated.
 *
 *  @param aMessage The message to be updated.
 *  @param pError   The error information if the method fails: Error.
 *
 */
- (void)updateMessageChange:(AgoraChatMessage *_Nonnull)aMessage
                      error:(AgoraChatError ** _Nullable)pError;

/**
 *  
 *  Marks a message as read.
 *
 *  @param aMessageId   The message ID.
 *  @param pError       The error information if the method fails: Error.
 *
 */
- (void)markMessageAsReadWithId:(NSString *_Nonnull)aMessageId
                          error:(AgoraChatError ** _Nullable)pError;

/**
 *  
 *  Marks all messages as read.
 *
 *  @param pError   The error information if the method fails: Error.
 *
 */
- (void)markAllMessagesAsRead:(AgoraChatError ** _Nullable)pError;


#pragma mark - Load Messages Methods

/**
 *  
 *  Gets a message with the ID.
 *
 *  @param aMessageId       The message ID.
 *  @param pError           The error information if the method fails: Error.
 *
 */
- (AgoraChatMessage * _Nullable)loadMessageWithId:(NSString * _Nonnull)aMessageId
                           error:(AgoraChatError ** _Nullable)pError;

/**
 *  
 *  Loads messages starting from the specified message id from local database. 
 * 
 *  This method returns messages in the sequence of the timestamp when they are received.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aMessageId       The specified message ID.
 *  @param aCount           The number of messages to load. If you set this parameter less than 1, it will be handled as count=1.
 *  @param aDirection       The message search direction: AgoraChatMessageSearchDirection.
                            AgoraChatMessageSearchDirectionUp: get aCount of messages before aMessageId;
                            AgoraChatMessageSearchDirectionDown: get aCount of messages after aMessageId
 *
 *  @result AgoraChatMessage  The message instance.
 *
 */
- (NSArray<AgoraChatMessage *> * _Nullable)loadMessagesStartFromId:(NSString * _Nullable)aMessageId
                          count:(int)aCount
                searchDirection:(AgoraChatMessageSearchDirection)aDirection;

/**
 *  
 *  Loads messages starting from the specified message ID from local database.
 * 
 *  Returning messages are sorted by receiving timestamp based on AgoraChatMessageSearchDirection. If the aMessageId is nil, will return starting from the latest message.
 *
 *  @param aMessageId       The specified message ID loading messages start from.
 *  @param aCount           The number of messages to load. If you set this parameter less than 1, it will be handled as count=1.
 *  @param aDirection       The message search direction: AgoraChatMessageSearchDirection. 
                            AgoraChatMessageSearchDirectionUp: get aCount of messages before the timestamp of the specified message ID; 
                            AgoraChatMessageSearchDirectionDown: get aCount of messages after the timestamp of the specified message ID.
 *  @param aCompletionBlock The completion block, which contains the error message if the method fails.
 *
 */
- (void)loadMessagesStartFromId:(NSString * _Nullable)aMessageId
                          count:(int)aCount
                searchDirection:(AgoraChatMessageSearchDirection)aDirection
                     completion:(void (^ _Nullable)(NSArray<AgoraChatMessage *> * _Nullable aMessages, AgoraChatError * _Nullable aError))aCompletionBlock;

/**
 *  
 *  Loads messages with specified message type from local database. Returning messages are sorted by receiving timestamp based on AgoraChatMessageSearchDirection.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aType            The message type to load. Type includes txt : text msg, img : image msg, loc: location msg, audio: audio msg, video: video msg, file: file msg, cmd:  command msg.
 *  @param aTimestamp       The reference timestamp for the messages to be loaded. If you set this parameter as a negative value, the SDK loads messages from the latest.
 *  @param aCount           The number of messages to load. If you set this parameter less than 1, it will be handled as count=1.
 *  @param aUsername        The user that sends the message. Setting it as NIL means that the SDK ignores this parameter.
 *  @param aDirection       The message search direction: AgoraChatMessageSearchDirection.
                            AgoraChatMessageSearchDirectionUp: get aCount of messages before aMessageId;
                            AgoraChatMessageSearchDirectionDown: get aCount of messages after aMessageId
 *
 *  @result AgoraChatMessage  The message instance.
 *
 */
- (NSArray<AgoraChatMessage *> * _Nullable)loadMessagesWithType:(AgoraChatMessageBodyType)aType
                   timestamp:(long long)aTimestamp
                       count:(int)aCount
                    fromUser:(NSString* _Nullable)aUsername
             searchDirection:(AgoraChatMessageSearchDirection)aDirection;

/**
 *  
 *  Loads messages with specified message type from local database. Returning messages are sorted by receiving timestamp based on AgoraChatMessageSearchDirection.
 *
 *  @param aType            The message type to load. Type includes txt : text msg, img : image msg, loc: location msg, audio: audio msg, video: video msg, file: file msg, cmd:  command msg.
 *  @param aTimestamp       The reference timestamp for the messages to be loaded. If you set this parameter as a negative value, the SDK loads messages from the latest.
 *  @param aCount           The number of messages to load. If you set this parameter less than 1, it will be handled as count=1.
 *  @param aUsername        The message sender (optional). If the parameter is nil, ignore.
 *  @param aDirection       The message search direction.
                            AgoraChatMessageSearchDirectionUp: get aCount of messages before aMessageId;
                            AgoraChatMessageSearchDirectionDown: get aCount of messages after aMessageId
 *  @param aCompletionBlock The completion block, which contains the error message if the method fails.
 *
 */
- (void)loadMessagesWithType:(AgoraChatMessageBodyType)aType
                   timestamp:(long long)aTimestamp
                       count:(int)aCount
                    fromUser:(NSString* _Nullable)aUsername
             searchDirection:(AgoraChatMessageSearchDirection)aDirection
                  completion:(void (^ _Nullable)(NSArray<AgoraChatMessage *> * _Nullable aMessages, AgoraChatError * _Nullable aError))aCompletionBlock;

/**
 *  
 *  Loads messages with specified keyword from local database, returning messages are sorted by receiving timestamp based on AgoraChatMessageSearchDirection. If reference timestamp is negative, load from the latest messages; if message count is negative, will be handled as count=1
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aKeyword         The keyword for searching the messages. Setting it as NIL means that the SDK ignores this parameter.
 *  @param aTimestamp       The reference timestamp for the messages to be loaded. If you set this parameter as a negative value, the SDK loads messages from the latest.
 *  @param aCount           The number of messages to load. If you set this parameter less than 1, it will be handled as count=1.
 *  @param aSender          The message sender (optional). If the parameter is nil, ignore.
 *  @param aDirection       The message search direction: AgoraChatMessageSearchDirection.
                            AgoraChatMessageSearchDirectionUp: get aCount of messages before aMessageId;
                            AgoraChatMessageSearchDirectionDown: get aCount of messages after aMessageId *  ----
 *
 *  @result AgoraChatMessage  The message list.
 *
 */
- (NSArray<AgoraChatMessage *> * _Nullable)loadMessagesWithKeyword:(NSString* _Nullable)aKeyword
                      timestamp:(long long)aTimestamp
                          count:(int)aCount
                       fromUser:(NSString* _Nullable)aSender
                searchDirection:(AgoraChatMessageSearchDirection)aDirection;

/**
 *  
 *  Loads messages with specified keyword from local database. Returning messages are sorted by receiving timestamp based on AgoraChatMessageSearchDirection. If reference timestamp is negative, load from the latest messages; if message count is negative, will be handled as count=1
 *
 *  @param aKeyword         The search keyword. If aKeyword=nil, ignore.
 *  @param aTimestamp       The load based on reference timestamp. If aTimestamp=-1, will load from the most recent (the latest) message
 *  @param aCount           The number of messages to load. If you set this parameter less than 1, it will be handled as count=1.
 *  @param aSender          The message sender (optional). If the parameter is nil, ignore.
 *  @param aDirection       The message search direction.
                            AgoraChatMessageSearchDirectionUp: get aCount of messages before aMessageId;
                            AgoraChatMessageSearchDirectionDown: get aCount of messages after aMessageId *  ----
 *  @param aCompletionBlock The completion block, which contains the error message if the method fails.
 *
 */
- (void)loadMessagesWithKeyword:(NSString* _Nullable)aKeyword
                      timestamp:(long long)aTimestamp
                          count:(int)aCount
                       fromUser:(NSString* _Nullable)aSender
                searchDirection:(AgoraChatMessageSearchDirection)aDirection
                     completion:(void (^ _Nullable)(NSArray<AgoraChatMessage *> * _Nullable aMessages, AgoraChatError * _Nullable aError))aCompletionBlock;

/**
 *  
 *  Loads custom messages with specified keyword from local database. The returning messages are sorted by receiving timestamp based on AgoraChatMessageSearchDirection. If the reference timestamp is negative, load from the latest messages; if message count is negative, will be handled as count=1.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aKeyword         The search keyword. If aKeyword=nil, ignore.
 *  @param aTimestamp       The load based on reference timestamp. If aTimestamp=-1, will load from the most recent (the latest) message
 *  @param aCount           The number of messages to load. If you set this parameter less than 1, it will be handled as count=1.
 *  @param aSender          The user that sends the message. Setting it as NIL means that the SDK ignores this parameter.
 *  @param aDirection       The message search direction.
                            AgoraChatMessageSearchDirectionUp: get aCount of messages before aMessageId;
                            AgoraChatMessageSearchDirectionDown: get aCount of messages after aMessageId *  ----
 *
 *  @result AgoraChatMessage  The message list.
 *
 */
- (NSArray<AgoraChatMessage *> * _Nullable)loadCustomMsgWithKeyword:(NSString*)aKeyword
                       timestamp:(long long)aTimestamp
                           count:(int)aCount
                        fromUser:(NSString* _Nullable)aSender
                 searchDirection:(AgoraChatMessageSearchDirection)aDirection;

/**
 *  
 *  Loads custom messages with specified keyword from local database. returning messages are sorted by receiving timestamp based on AgoraChatMessageSearchDirection. If reference timestamp is negative, load from the latest messages; if message count is negative, will be handled as count=1
 *
 *  @param aKeyword         The keyword for searching the messages. Setting it as NIL means that the SDK ignores this parameter.
 *  @param aTimestamp       The reference Unix timestamp for the messages to be loaded. If you set this parameter as a negative value, the SDK loads messages from the latest.
 *  @param aCount           The number of messages to load. If you set this parameter less than 1, it will be handled as count=1.
 *  @param aSender          The user that sends the message. Setting it as NIL means that the SDK ignores this parameter.
 *  @param aDirection       The message search direction: AgoraChatMessageSearchDirection.
                            AgoraChatMessageSearchDirectionUp: get aCount of messages before aMessageId;
                            AgoraChatMessageSearchDirectionDown: get aCount of messages after aMessageId *  ----
 *  @param aCompletionBlock The completion block, which contains the error message if the method fails.
 *
 */
- (void)loadCustomMsgWithKeyword:(NSString* _Nullable)aKeyword
                       timestamp:(long long)aTimestamp
                           count:(int)aCount
                        fromUser:(NSString* _Nullable)aSender
                 searchDirection:(AgoraChatMessageSearchDirection)aDirection
                      completion:(void (^ _Nullable)(NSArray<AgoraChatMessage *> * _Nullable aMessages, AgoraChatError * _Nullable aError))aCompletionBlock;

/**
 *  
 *  Loads messages within specified time range from local database. Returning messages are sorted by sending timestamp.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aStartTimestamp  The starting timestamp in miliseconds.
 *  @param aEndTimestamp    The ending timestamp in miliseconds.
 *  @param aCount           The number of messages to load. If you set this parameter less than 1, it will be handled as count=1.
 *
 *  @result AgoraChatMessage       The message list.
 *
 */
- (NSArray<AgoraChatMessage *> * _Nullable)loadMessagesFrom:(long long)aStartTimestamp
                      to:(long long)aEndTimestamp
                   count:(int)aCount;

/**
 *  
 *  Loads messages within specified time range from local database. Returning messages are sorted by sending timestamp.
 *
 *  @param aStartTimestamp  The starting timestamp in miliseconds.
 *  @param aEndTimestamp    The ending timestamp in miliseconds.
 *  @param aCount           The number of messages to load. If you set this parameter less than 1, it will be handled as count=1.
 *  @param aCompletionBlock The completion block, which contains the error message if the method fails.
 *
 */
- (void)loadMessagesFrom:(long long)aStartTimestamp
                      to:(long long)aEndTimestamp
                   count:(int)aCount
              completion:(void (^ _Nullable)(NSArray<AgoraChatMessage *> * _Nullable aMessages, AgoraChatError * _Nullable aError))aCompletionBlock;

@end
