/**
 *  
 *  @header AgoraChatMessage.h
 *  @abstract Chat message
 *  @author Hyphenate
 *  @version 3.00
 */

#import <Foundation/Foundation.h>

#import "AgoraChatMessageBody.h"
#import "AgoraChatMessageReaction.h"

/**
 *  
 *  Chat types.
 */
typedef NS_ENUM(NSInteger, AgoraChatType) {
    AgoraChatTypeChat   = 0,   /**  One-to-one chat. */
    AgoraChatTypeGroupChat,    /**  Group chat. */
    AgoraChatTypeChatRoom,     /**  Chatroom. */
};

/**
 *  
 *  The message delivery status types.
 */
typedef NS_ENUM(NSInteger, AgoraChatMessageStatus) {
    AgoraChatMessageStatusPending  = 0,    /**  The message delivery is pending.*/
    AgoraChatMessageStatusDelivering,      /**  The message is being delivered.*/
    AgoraChatMessageStatusSucceed,         /**  The message is successfully delivered.*/
    AgoraChatMessageStatusFailed,          /**  The message fails to be delivered.*/
};

/**
 *  
 *  The message direction types.
 */
typedef NS_ENUM(NSInteger, AgoraChatMessageDirection) {
    AgoraChatMessageDirectionSend = 0,    /**  This message is sent from the local client.*/
    AgoraChatMessageDirectionReceive,     /**  The message is received by the local client.*/
};
/**
 *  
 *  The chat message class.
 */
@class AgoraChatThread;
@interface AgoraChatMessage : NSObject

/**
 *  
 *  The message ID, which is the unique identifier of the message.
 */
@property (nonatomic, copy) NSString * _Nonnull messageId;

/**
 *  
 *  The conversation ID, which is the unique identifier of the conversation.
 */
@property (nonatomic, copy) NSString * _Nonnull conversationId;

/**
 *  
 *  The message delivery direction.
 */
@property (nonatomic) AgoraChatMessageDirection direction;

/**
 *  
 *  The user sending the message.
 */
@property (nonatomic, copy) NSString * _Nonnull from;

/**
 *  
 *  The user receiving the message.
 */
@property (nonatomic, copy) NSString * _Nonnull to;

/**
 *  
 *  The Unix timestamp for the chat server receiving the message. The unit is second.
 */
@property (nonatomic) long long timestamp;

/**
 *  
 *  The Unix timestamp for the local client sending or receiving the message.
 */
@property (nonatomic) long long localTime;

/**
 *  
 *  The chat type.
 */
@property (nonatomic) AgoraChatType chatType;

/**
 *  
 *  The message delivery status. See {@link AgoraChatMessageStatus}.
 */
@property (nonatomic) AgoraChatMessageStatus status;

/*!
 *   (Local database does not store. The default value for reading or pulling roaming messages from the database is YES)
 *  Message Online Status
 */
@property (nonatomic, readonly) BOOL onlineState;

/**
 *  
 *  Whether the message read receipt (from the message sender) is sent or received (by the message recipient).
 *
 *  - `Yes`: Yes;
 *  - `No`: No.
 */
@property (nonatomic) BOOL isReadAcked;

/**
 *  
 *  Is it a message sent within a thread
 */
@property (nonatomic) BOOL isChatThreadMessage;

/**
 *  
 *  Whether read receipts are required for group messages.
 *
 *  - `Yes`: Yes;
 *  - `No`: No.
 */
@property (nonatomic) BOOL isNeedGroupAck;

/**
 *  
 *  The number of read receipts for group messages.
 */
@property (nonatomic, readonly) int groupAckCount;

/**
 *  
 *  For the message sender, this property indicates whether the delivery receipt is received.
 *
 *      - For the message sender, this property indicates whether the delivery receipt is received.
 *      - For the message recipient, this property indicates whether the delivery receipt is sent.
 *
 *  If you set `enableDeliveryAck` in AgoraChatOptions as `YES`, the SDK automatically sends the delivery receipt after receiving a message.
 */
@property (nonatomic) BOOL isDeliverAcked;

/**
 *  
 *  Whether the message is read.
 *
 *  - `Yes`: Yes;
 *  - `No`: No.
 */
@property (nonatomic) BOOL isRead;

/**
 *  
 *  Whether the voice message is played.
 *
 *  - `Yes`: Yes;
 *  - `No`: No.
 */
@property (nonatomic) BOOL isListened;

/**
 *  
 *  The message body.
 */
@property (nonatomic, strong) AgoraChatMessageBody * _Nonnull body;

/**
 *  
 *  The Reaction list.
 */
@property (nonatomic, readonly) NSArray <AgoraChatMessageReaction *>* _Nullable reactionList;

/**
 *  
 *  Gets the Reaction content by the Reaction ID.
 *
 *  @param reaction   The Reaction ID.
 *
 *  @result    The Reaction content.
 */
- (AgoraChatMessageReaction *_Nullable)getReaction:(NSString * _Nonnull)reaction;

/**
 *  
 *  The custom message extension.
 *
 *  This member is in the key-value format, where the key is the extension field name of the NSString type, and the value must be of the NSString or NSNumber(Bool, Int, unsigned int, long long, double) type.
 */
@property (nonatomic, copy) NSDictionary * _Nullable ext;
/**
 *  
 *  Get an overview of the thread in the message (currently only supported by group messages)
 */

@property (readonly) AgoraChatThread * _Nullable chatThread;

/**
 *  
 *  Initializes a message instance.
 *
 *  @param aConversationId   The conversation ID.
 *  @param aFrom           The user that sends the message.
 *  @param aTo         The user that receives the message.
 *  @param aBody             The message body.
 *  @param aExt              The message extention.
 *
 *  @result    The message instance.
 */


- (id _Nonnull )initWithConversationID:(NSString *_Nonnull)aConversationId
                                  from:(NSString *_Nonnull)aFrom
                                    to:(NSString *_Nonnull)aTo
                                  body:(AgoraChatMessageBody *_Nonnull)aBody
                                   ext:(NSDictionary *_Nullable)aExt;

/**
 *  
 *  Initializes a message instance.
 *
 *  @param aConversationId   The conversation ID.
 *  @param aBody             The message body.
 *  @param aExt              The message extention.
 *
 *  @result    The message instance.
 */


- (id _Nonnull )initWithConversationID:(NSString *_Nonnull)aConversationId
                                  body:(AgoraChatMessageBody *_Nonnull)aBody
                                   ext:(NSDictionary *_Nullable)aExt;

@end
