/**
 *  
 *  @header AgoraChatOptions.h
 *  @abstract SDK setting options
 *  @author Hyphenate
 *  @version 3.00
 */

#import <Foundation/Foundation.h>

#import "AgoraChatCommonDefs.h"

/**
 *  
 *  Log output level types.
 */
typedef NS_ENUM(NSInteger, AgoraChatLogLevel)
{
    AgoraChatLogLevelDebug = 0, /**  Output all logs. */
    AgoraChatLogLevelWarning,   /**  Output warnings and errors. */
    AgoraChatLogLevelError      /**  Output errors only. */
} ;

/**
 *  
 *  The SDK setting options.
 */
@interface AgoraChatOptions : NSObject

/**
 *  
 *  The app key, which is the unique identifier of the project.
 */
@property(nonatomic, copy, readonly) NSString *appkey;

/**
 *  
 *  Whether to print logs on Console. The default value is `NO`.
 */
@property(nonatomic, assign) BOOL enableConsoleLog;

/**
 *  
 *  The log output level. The default value is AgoraChatLogLevelDebug, which means all log levels.
 */
@property(nonatomic, assign) AgoraChatLogLevel logLevel;

/**
 *  
 *  Whether to only use the HTTPS protocol. The default value is `NO`.
 */
@property(nonatomic, assign) BOOL usingHttpsOnly;

/**
 *  
 *  Whether to enable automatic login. The default value is `YES`.
 *
 *  You need to set this parameter before the SDK is initialized.
 */
@property(nonatomic, assign) BOOL isAutoLogin;

/**
 *  
 *  Whether to delete all the group messages when leaving the group. The default value is YES.
 */
@property(nonatomic, assign) BOOL deleteMessagesOnLeaveGroup;

/**
 *  
 *  Whether to delete all the chat room messages when leaving the chat room. The default value is YES.
 */
@property(nonatomic, assign) BOOL deleteMessagesOnLeaveChatroom;

/**
 *  
 *  Whether to allow the chatroom owner leave the room. The default value is YES.
 */
@property(nonatomic, assign) BOOL canChatroomOwnerLeave;

/**
 *  
 *  Whether to automatically accept group invitation. The default value is `YES`.
 */
@property(nonatomic, assign) BOOL autoAcceptGroupInvitation;

/**
 *  
 *  Whether to automatically approve contact request. The default value is `NO`.
 */
@property(nonatomic, assign) BOOL autoAcceptFriendInvitation;

/**
 *  
 *  Whether to automatically download image or video thumbnails and voice messages. The default value is `YES`.
 */
@property(nonatomic, assign) BOOL autoDownloadThumbnail;

/**
 * 
 * Whether need to receive the message read receipt. The default value is `YES`.
 */
@property(nonatomic, assign) BOOL enableRequireReadAck;
/**
 *  
 *  Whether to send the message delivery receipt. The default value is `NO`. If you set it to `YES`, the SDK automatically send a delivery receipt when you receive a chat message.
 */
@property(nonatomic, assign) BOOL enableDeliveryAck;

/**
 *  
 *  Whether to sort messages by server received time when loading message from database. The default value is `YES`.
 */
@property(nonatomic, assign) BOOL sortMessageByServerTime;

/**
 *  
 *  Whether to automatically upload or download the attachment in the message. The default value is `YES`.
 */
@property(nonatomic, assign) BOOL isAutoTransferMessageAttachments;

/**
 *  
 *  Whether to enable fpa feature. The default value is `NO`.
 */
@property(nonatomic, assign) BOOL enableFpa;

/**
 *  
 *  The certificate name of Apple Push Notification Service.
 *
 *  Ensure that you set this parameter when calling `initializeSDKWithOptions`. During the app runtime, you can not change the settings.
 */
@property(nonatomic, copy) NSString *apnsCertName;

/**
 *  
 *  The certificate name of Apple PushKit Service.
 *
 *  Ensure that you set this parameter when calling `initializeSDKWithOptions`. During the app runtime, you can not change the settings.
 */
@property(nonatomic, copy) NSString *pushKitCertName;

/**
 *  
 *  Gets a SDK setting options instance.
 *
 *  @param aAppkey  The app key.
 *
 *  @result  The SDK setting options instance.
 */
+ (instancetype _Nonnull)optionsWithAppkey:(NSString * _Nonnull)aAppkey;

#pragma mark - EM_DEPRECATED_IOS 3.8.8
/**
 *  
 *  Whether to delete all the group messages when leaving the group. The default value is YES.
 */
@property(nonatomic, assign) BOOL isDeleteMessagesWhenExitGroup __deprecated_msg("Use deleteMessagesOnLeaveGroup instead");

/**
 *  
 *  Whether to delete all the chat room messages when leaving the chat room. The default value is YES.
 */
@property(nonatomic, assign) BOOL isDeleteMessagesWhenExitChatRoom
    __deprecated_msg("Use deleteMessagesOnLeaveChatroom instead");

/**
 *  
 *  if allow chat room's owner can leave the chat room. The default value is YES.
 */
@property(nonatomic, assign) BOOL isChatroomOwnerLeaveAllowed
    __deprecated_msg("Use canChatroomOwnerLeave instead");

/**
 *  
 *  Whether to automatically accept group invitation. The default value is YES.
 */
@property(nonatomic, assign) BOOL isAutoAcceptGroupInvitation
    __deprecated_msg("Use autoAcceptGroupInvitation instead");

/**
 *  
 *  Whether to automatically approve friend request. The default value is NO.
 */
@property(nonatomic, assign) BOOL isAutoAcceptFriendInvitation
    __deprecated_msg("Use autoAcceptFriendInvitation instead");

/**
 *  
 *  Whether to automatically download thumbnail of image&video and audio. The default value is YES.
 */
@property(nonatomic, assign) BOOL isAutoDownloadThumbnail
    __deprecated_msg("Use autoDownloadThumbnail instead");

@end
