/**
 *  @header AgoraChatClient.h
 *  @abstract SDK Client
 *  @author Hyphenate
 *  @version 3.00
 */

#import <Foundation/Foundation.h>

#import "AgoraChatClientDelegate.h"
#import "AgoraChatError.h"
#import "AgoraChatMultiDevicesDelegate.h"
#import "AgoraChatOptions.h"
#import "AgoraChatPushOptions.h"

#import "IAgoraChatManager.h"
#import "IAgoraChatroomManager.h"
#import "IAgoraChatContactManager.h"
#import "IAgoraChatGroupManager.h"
#import "IAgoraChatThreadManager.h"
#import "IAgoraChatPushManager.h"
#import "IAgoraChatUserInfoManager.h"
#import "IAgoraChatTranslateManager.h"
#import "IAgoraChatPresenceManager.h"

#import "AgoraChatDeviceConfig.h"
#import "AgoraChatLocalNotificationManager.h"
#import "AgoraChatTranslationResult.h"
#import "AgoraChatLogDelegate.h"

/**
 *  
 *  This enum lists the server diagnostic tests which are run for the current user when you call AgoraChatClient::serviceCheckWithUsername. This is an asynchronous method. If there is an error, check the error code to know the error information.
 */
typedef NS_ENUM(NSInteger, AgoraChatServerCheckType) {
  AgoraChatServerCheckAccountValidation = 0, /**  The check of the account validity. */
  AgoraChatServerCheckGetDNSListFromServer,  /**  The check of getting DNS from the server. */
  AgoraChatServerCheckGetTokenFromServer,    /**  The check of getting the token from the server. */
  AgoraChatServerCheckDoLogin,               /**  The check of the login mode. */
  AgoraChatServerCheckDoLogout,              /**  The check of the logout mode. */
};

/**
 * 
 *  The AgoraChatClient, which is the entry point of the Chat SDK. You can log in, log out, and access other functionalities such as group and chatroom with this class.
 *  [AgoraChatClient sharedClient].groupManager;
 */
@interface AgoraChatClient : NSObject
{
  AgoraChatPushOptions *_pushOptions;
}

/**
 *  
 *  The SDK version number.
 */
@property(nonatomic, strong, readonly) NSString * _Nonnull version;

/**
 *  
 *  The ID of the user currently logged into your chat app.
 */
@property(nonatomic, strong, readonly) NSString * _Nullable currentUsername;

/**
 *  
 *  The SDK setting options. For example, whether to use https by default.
 */
@property(nonatomic, strong, readonly) AgoraChatOptions * _Nonnull options;

/**
 *  
 *  The chat manager module.
 */
@property(nonatomic, strong, readonly) id<IAgoraChatManager> _Nullable chatManager;

/**
 *  
 *  The contact manager module.
 */
@property(nonatomic, strong, readonly) id<IAgoraChatContactManager> _Nullable contactManager;

/**
 *  
 *  The group manager module.
 */
@property(nonatomic, strong, readonly) id<IAgoraChatGroupManager> _Nullable groupManager;

/**
 *  
 *  Thread Management
 */
@property (nonatomic, strong, readonly) id<IAgoraChatThreadManager> _Nullable threadManager;

/*!
 *  
 *  The chat room manager module.
 */
@property(nonatomic, strong, readonly) id<IAgoraChatroomManager> _Nullable roomManager;

/**
 *  
 *   The push manager module.
 */
@property (nonatomic, strong, readonly) id<IAgoraChatPushManager> _Nullable pushManager;

/**
 *  
 *  Whether to let a user automatically log in to the chat server with the username used in the previous session. If the login fails, for example, because of a wrong password or the username deactivated, the isAutoLogin parameter is reset to NO, and you need to set it back to YES to allow automatic login.
 *
 */
@property(nonatomic, readonly) BOOL isAutoLogin;

/**
 *  
 *  Returns `true` if the current user is logged in.
 */
@property(nonatomic, readonly) BOOL isLoggedIn;

/**
 *  
 *  Whether the SDK is connected to the chat server.
 */
@property(nonatomic, readonly) BOOL isConnected;

/**
 *  
 *  The token for accessing the current chat.
 */
@property(nonatomic, readonly) NSString * _Nullable accessUserToken;

/**
 *  
 *  The user attributes manager module.
 */
@property(nonatomic, strong, readonly) id<IAgoraChatUserInfoManager> _Nullable userInfoManager;

/**
 *  
 *  The presence  manager module.
 */
@property (nonatomic, strong, readonly) id<IAgoraChatPresenceManager> _Nullable presenceManager;

/**
 *  
 *  Creates a Client instance. The Client class is the entry to the Chat SDK. You need to call this method to create a Client instance before calling any other method.
 */
+ (instancetype _Nonnull )sharedClient;


/**
 *  
 *  The SDK version number.
 */
- (NSString *)version;

#pragma mark - Delegate

/**
 *  
 *  Add the AgoraChatClientDelegate, so when the delegate event occurs, the delegate will call the methods automatically in <AgoraChatClientDelegate>.
 *
 *  @param aDelegate  The delegate that you want to add: ClientDelegate.
 *  @param aQueue     (optional) The queue of calling delegate methods. If you want to run the app on the main thread, set this parameter as nil. */
- (void)addDelegate:(id<AgoraChatClientDelegate>_Nonnull)aDelegate
      delegateQueue:(dispatch_queue_t _Nullable )aQueue;

/**
 *  
 *  Remove the delegate.
 *
 *  @param aDelegate  The delegate that you want to remove.
 */
- (void)removeDelegate:(id _Nonnull )aDelegate;

/**
 *  
 *  Add the multi-device delegate.
 *
 *  @param aDelegate  The delegate that you want to add: MultiDevicesDelegate.
 *  @param aQueue     The queue of calling delegate methods.
 */
- (void)addMultiDevicesDelegate:(id<AgoraChatMultiDevicesDelegate>_Nonnull)aDelegate
                  delegateQueue:(dispatch_queue_t _Nullable )aQueue NS_SWIFT_NAME(addMultiDevices(delegate:queue:));

/**
 *  
 *  Remove the multi-device delegate.
 *
 *  @param aDelegate  The multi-device delegate that you want to delete.
 */
- (void)removeMultiDevicesDelegate:(id<AgoraChatMultiDevicesDelegate>_Nonnull)aDelegate;

#pragma mark - Initialize SDK

/**
 *  
 *  Initialize the SDK.
 *
 *  @param aOptions   The SDK setting options: Options..
 *
 *  @Result AgoraChatError   A description of the issue that caused this call to fail.
 */
- (AgoraChatError *_Nullable)initializeSDKWithOptions:(AgoraChatOptions * _Nonnull )aOptions;

#pragma mark - Change AppKey

/**
 *  
 *  Update the unique identifier used to access server.
 *
 *  You retrieve the new app key from server. As this key controls all access to server for your app, you can only update the key when the current user is logged out.
 *
 *  @param aAppkey  The app key.
 *
 *  @Result A description of the issue that caused this call to fail.
 */
- (AgoraChatError *_Nullable)changeAppkey:(NSString * _Nonnull)aAppkey;

#pragma mark - User Registeration

/**
 *  
 *  Register a new user with your chat network.
 *
 *  After you call `initializeSDKWithOptions` and register your app in console, the app has access to all the features registered inside your chat network. You add and remove users inside your chat network; depending on how you implement your app, you control the people each user can see inside your network.
 *
 *  This is a synchronous method and blocks the current thread. To ensure registration reliability, we recommend using the REST API to register new chat users.
 *
 *  @param aUsername  The username. The maximum length is 64 characters. Ensure that you set this parameter. Supported characters include the 26 English letters (a-z), the ten numbers (0-9), the underscore (_), the hyphen (-), and the English period (.). This parameter is case insensitive, and upper-case letters are automatically changed to low-case ones. If you want to set this parameter as a regular expression, set it as ^[a-zA-Z0-9_-]+$.
 *  @param aPassword   The password. The maximum length is 64 characters. Ensure that you set this parameter.
 *
 *  @result A description of the issue that caused this call to fail.
 */
- (AgoraChatError *_Nullable)registerWithUsername:(NSString * _Nonnull)aUsername
                                  password:(NSString * _Nonnull)aPassword;

/**
 *  
 *  Register a new user with your chat network.
 *
 *  This is an asynchronous method.
 *
 *  After you call `initializeSDKWithOptions` and register your app in console, the app has access to all the features registered inside your chat network. You add and remove users inside your chat network; depending on how you implement your app, you control the people each user can see inside your network.
 *
 *  To ensure registration reliability, we recommend using the REST API to register new chat users.
 *
 *  @param aUsername        The username. The maximum length is 64 characters. Ensure that you set this parameter. Supported characters include the 26 English letters (a-z), the ten numbers (0-9), the underscore (_), the hyphen (-), and the English period (.). This parameter is case insensitive, and upper-case letters are automatically changed to low-case ones. If you want to set this parameter as a regular expression, set it as ^[a-zA-Z0-9_-]+$.
 *  @param aPassword        The password. The maximum length is 64 characters. Ensure that you set this parameter.
 *  @param aCompletionBlock The completion block, which contains the username and the error message if the method fails.
 *
 */
- (void)registerWithUsername:(NSString *_Nonnull)aUsername
                    password:(NSString *_Nonnull)aPassword
                  completion:(void (^_Nullable)(NSString * _Nonnull aUsername, AgoraChatError * _Nullable aError))aCompletionBlock;

#pragma mark - Login

/**
 *  
 *  Fetches the token from the server.
 *
 *  This is an asynchronous method.
 *
 *  @param aUsername     The username.
 *  @param aPassword     The password.
 *  @param aCompletionBlock The completion block, which contains the token and the error message if the method fails.
 *
 */
- (void)fetchTokenWithUsername:(NSString *_Nonnull)aUsername
                      password:(NSString *_Nonnull)aPassword
                    completion:(void (^_Nullable)(NSString * _Nullable aToken, AgoraChatError * _Nullable aError))aCompletionBlock;

/**
 *  
 *  A user logs in to the chat server with a password.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aUsername  The username. The maximum length is 64 characters. Ensure that you set this parameter. Supported characters include the 26 English letters (a-z), the ten numbers (0-9), the underscore (_), the hyphen (-), and the English period (.). This parameter is case insensitive, and upper-case letters are automatically changed to low-case ones. If you want to set this parameter as a regular expression, set it as ^[a-zA-Z0-9_-]+$.
 *  @param aPassword  The password. The maximum length is 64 characters. Ensure that you set this parameter.
 *
 *  @result Returns nil on success, and the description of the issue that cause the call to fail.
 */
- (AgoraChatError *_Nullable)loginWithUsername:(NSString *_Nonnull)aUsername
                               password:(NSString *_Nonnull)aPassword;

/**
 *  
 *  A user logs in to the chat server with a password.
 *
 *  This is an asynchronous method.
 *
 *  @param aUsername        The username. The maximum length is 64 characters. Ensure that you set this parameter. Supported characters include the 26 English letters (a-z), the ten numbers (0-9), the underscore (_), the hyphen (-), and the English period (.). This parameter is case insensitive, and upper-case letters are automatically changed to low-case ones. If you want to set this parameter as a regular expression, set it as ^[a-zA-Z0-9_-]+$.
 *  @param aPassword        The password. The maximum length is 64 characters. Ensure that you set this parameter.
 *  @param aCompletionBlock The completion block, which contains the username and the error message if the method fails.
 *
 */
- (void)loginWithUsername:(NSString *_Nonnull)aUsername
                 password:(NSString *_Nonnull)aPassword
               completion:(void (^_Nullable)(NSString * _Nonnull aUsername, AgoraChatError *_Nullable aError))aCompletionBlock;

/**
 *  
 *  A user logs in to the chat server with a token. This method does not support automatic isLoggedIn.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aUsername  The username. The maximum length is 64 characters. Ensure that you set this parameter. Supported characters include the 26 English letters (a-z), the ten numbers (0-9), the underscore (_), the hyphen (-), and the English period (.). This parameter is case insensitive, and upper-case letters are automatically changed to low-case ones. If you want to set this parameter as a regular expression, set it as ^[a-zA-Z0-9_-]+$.
 *  @param aToken     The token for user logging in to Chat server.
 *
 *  @result Returns nil on success, and the description of the issue that cause the call to fail.
 */
- (AgoraChatError *_Nullable)loginWithUsername:(NSString *_Nonnull)aUsername
                                  token:(NSString *_Nonnull)aToken;

/**
 *  
 *  A user logs in to the chat server with a token. This method support automatic login.
 *
 *  This is an asynchronous method.
 *
 *  @param aUsername        The username. The maximum length is 64 characters. Ensure that you set this parameter. Supported characters include the 26 English letters (a-z), the ten numbers (0-9), the underscore (_), the hyphen (-), and the English period (.). This parameter is case insensitive, and upper-case letters are automatically changed to low-case ones. If you want to set this parameter as a regular expression, set it as ^[a-zA-Z0-9_-]+$.
 *  @param aToken           The token for logging in to the chat server.
 *  @param aCompletionBlock The completion block, which contains the username and the error message if the method fails.
 *
 */
- (void)loginWithUsername:(NSString *_Nonnull)aUsername
                    token:(NSString *_Nonnull)aToken
               completion:(void (^_Nullable)(NSString * _Nonnull aUsername, AgoraChatError *_Nullable aError))aCompletionBlock;

/**
 *  
 *  A user logs in to the chat server with Agora Chat user token. Supports automatic login.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aUsername    The username.
 *  @param aAgoraToken  The Agora Chat user token.
 *
 *  @result Returns nil on success, and the description of the issue that cause the call to fail.
 */
- (AgoraChatError *_Nullable)loginWithUsername:(NSString *_Nonnull)aUsername
                             agoraToken:(NSString *_Nonnull)aAgoraToken;

/**
 *  
 *  A user logs in to the chat server with Agora Chat user token. Supports automatic login.
 *
 *
 *  @param aUsername        The username.
 *  @param aAgoraToken      The Agora Chat user token.
 *  @param aCompletionBlock The callback of completion block, which contains the description of the cause to the issue if the method fails.
 *
 */
- (void)loginWithUsername:(NSString *_Nonnull)aUsername
               agoraToken:(NSString *_Nonnull)aAgoraToken
               completion:(void (^_Nullable)(NSString * _Nonnull aUsername, AgoraChatError *_Nullable aError))aCompletionBlock;

/**
 *  
 *  Renews the token when the current token expires.
 *
 *  The token expires after a period of time once the token schema is enabled when:
 *  - The SDK triggers the onTokenPrivilegeWillExpire callback, or
 *  - The onConnectionStateChanged callback reports the CONNECTION_CHANGED_TOKEN_EXPIRED(9) error.
 *
 *  The app should retrieve a new token from the server and call this method to renew it. Failure to do so results in the SDK disconnecting from the server.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param newAgoraToken The new Agora Chat token。
 *
 *  @result The result which contains the description of the cause to the failure if call fails.
 */
- (AgoraChatError *_Nullable)renewToken:(NSString *_Nonnull)newAgoraToken;

#pragma mark - Logout

/**
 *  
 *  A user logs out of the chat server.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aIsUnbindDeviceToken Whether to unbind the username from the device(Set to YES to unbind the user currently logged into the app from this device). That stops the user device receiving push notifications from the Apple Push Notifications service.
 *
 *  @result A description of the issue that caused this call to fail.
 */
- (AgoraChatError *_Nullable)logout:(BOOL)aIsUnbindDeviceToken;

/**
 *  
 *  A user logs out of the chat server.
 *
 *  This is an asynchronous method.
 *
 *  @param aIsUnbindDeviceToken     Whether to unbind the username from the device. If you unbind the username from the device (setting this parameter as YES) and logs out, the device no longer receives messages from the Apple Push Notification Service.
 *  @param aCompletionBlock         The completion block, which contains the token and the error message if the method fails.
 *
 */
- (void)logout:(BOOL)aIsUnbindDeviceToken
    completion:(void (^_Nullable)(AgoraChatError * _Nullable aError))aCompletionBlock;

#pragma mark - PushKit

/**
 *  
 *  Use the pushkit token to bind the user and the device, which is required to enable Apple PushKit Service.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aPushToken      The pushkit token to bind.
 *
 *  @result A description of the issue that caused this call to fail.
 */
- (AgoraChatError *_Nullable)bindPushKitToken:(NSData *_Nullable)aPushToken;

/**
 *  
 *  Register a pushkit token, this is required to enable Apple PushKit Service, which is for VOIP CALL.
 *
 *  This is an asynchronous method.
 *
 *  @param aPushToken           The pushkit token to bind.
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 */
- (void)registerPushKitToken:(NSData *_Nullable)aPushToken
                  completion:(void (^_Nullable)(AgoraChatError * _Nullable aError))aCompletionBlock;

/**
 *  
 *  Unbind the Apple PushKit token.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @result A description of the issue that caused this call to fail.
 */
- (AgoraChatError *_Nullable)unBindPushKitToken;

/**
 *  
 *  Unregister the Apple PushKit token.
 *
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 */
- (void)unRegisterPushKitTokenWithCompletion:(void (^_Nullable)(AgoraChatError * _Nullable aError))aCompletionBlock;

#pragma mark - APNs

/**
 *  
 *  Device token binding is required to enable Apple Push Notification Service.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aDeviceToken  Device token to bind.
 *
 *  @Result Returns nil on success, and the description of the issue that cause the call to fail.
 */
- (AgoraChatError *_Nullable)bindDeviceToken:(NSData *_Nonnull)aDeviceToken;

/**
 *  
 *  Device token binding is required to enable Apple push notification service.
 *
 *  @param aDeviceToken         The device token to bind.
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 */
- (void)registerForRemoteNotificationsWithDeviceToken:(NSData *_Nonnull)aDeviceToken
                                           completion:(void (^_Nullable)(AgoraChatError *_Nullable aError))aCompletionBlock;

#pragma mark - Log

/**
 *  
 *  Upload the log to the chat server.
 *  The information in the debug log is used by our engineers to fix errors and improve system performance.
 *  Make sure to use the `AgoraChatLog` class.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @result Returns nil on success, and error on failure which contains the description of the issue that cause the call to fail_Nullable.
 */
- (AgoraChatError *_Nullable)uploadLogToServer;

/**
 *  
 *  Upload debugging log to server.
 *
 *  @param aCompletionBlock     The completion block, which contains the token and the error message if the method fails.
 */
- (void)uploadDebugLogToServerWithCompletion:(void (^_Nullable)(AgoraChatError *_Nullable aError))aCompletionBlock;

/**
 *  
 *  Compress the debug log into a gzip archive.
 *  Best practice is to delete this debug archive as soon as it is no longer used.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param pError  A description of the issue that caused this call to fail.
 *
 *  @result NSString The full filepath to the debug archive.
 */
- (NSString *_Nullable)getLogFilesPath:(AgoraChatError **_Nullable)pError;

/**
 *  
 *  Compresses the log files in the format of .gz and returns the path of the compressed file. Recommends deleting the compressed file when it is no longer used.
 *
 *  This is an asynchronous method.
 *
 *  @param aCompletionBlock     The completion block, which contains the token and the error message if the method fails.
 */
- (void)getLogFilesPathWithCompletion:(void (^_Nullable)(NSString *_Nullable aPath, AgoraChatError * _Nullable aError))aCompletionBlock;

/**
 *  
 *  Output log info to log file. You can call this method after the SDK has been initialized.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aLog The log info.
 */
- (void)log:(NSString *_Nonnull)aLog;

/**
 *  
 *  Add the log callback delegate.
 *
 *  @param aDelegate  The delegate that you want to add: AgoraChatLogDelegate.
 *  @param aQueue     The queue of calling delegate methods.
 */
- (void)addLogDelegate:(id<AgoraChatLogDelegate>_Nonnull)aDelegate
         delegateQueue:(dispatch_queue_t _Nullable )aQueue NS_SWIFT_NAME(addLog(delegate:queue:));

/**
 *  
 *  Remove the log callback delegate.
 *
 *  @param aDelegate  The log callback delegate that you want to delete.
 */
- (void)removeLogDelegate:(id<AgoraChatLogDelegate>_Nonnull)aDelegate NS_SWIFT_NAME(removeLog(delegate:));

#pragma mark - Multi Devices

/**
 *  
 *  Retrieve the array of devices the user is currently logged into.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aUsername      The username.
 *  @param aPassword      The password.
 *  @param pError         A description of the issue that caused this call to fail.
 *
 *  @result  The information of the logged in devices, an array of <AgoraChatDeviceConfig> objects.
 */
- (NSArray<AgoraChatDeviceConfig*> *_Nullable)getLoggedInDevicesFromServerWithUsername:(NSString * _Nonnull)aUsername
                                                                       password:(NSString * _Nonnull)aPassword
                                                                          error:(AgoraChatError ** _Nullable)pError;

/**
 *  
 *  Get all the device information <AgoraChatDeviceConfig> that logged in to the server.
 *
 *  This is an asynchronous method.
 *
 *  @param aUsername     The username.
 *  @param aPassword     The password.
 *  @param aCompletionBlock     The completion block, which contains the list and the error message if the method fails.
 *
 */
- (void)getLoggedInDevicesFromServerWithUsername:(NSString *_Nonnull)aUsername
                                        password:(NSString *_Nonnull)aPassword
                                      completion:(void (^_Nullable)(NSArray<AgoraChatDeviceConfig*> * _Nullable aList, AgoraChatError *_Nullable aError))aCompletionBlock;

/**
 *  
 *  Kick a single user from your app installed on a specific device.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aUsername     The username.
 *  @param aPassword     The password.
 *  @param aResource     The device to log `aUsername` out from. Call `getLoggedInDevicesFromServerWithUsername` to retrieve the list of devices `aUsername` is currently logged into.
 *
 *  @result Returns nil on success, and the description of the issue that cause the call to fail.
 */
- (AgoraChatError *_Nullable)kickDeviceWithUsername:(NSString *_Nonnull)aUsername
                                    password:(NSString *_Nonnull)aPassword
                                    resource:(NSString *_Nonnull)aResource;

/**
 *  
 *  Kick a single user from your app installed on a specific device.
 *
 *  The device information can be obtained from `getLoggedInDevicesFromServerWithUsername`.
 *
 *  This is an asynchronous method.
 *
 *  @param aUsername     The username.
 *  @param aPassword     The password.
 *  @param aResource     The device to be logged out of. You can get the logged in devices through `getLoggedInDevicesFromServerWithUsername`.
 *  @param aCompletionBlock The completion block, which contains the error message if the method fails.
 */
- (void)kickDeviceWithUsername:(NSString *_Nonnull)aUsername
                      password:(NSString *_Nonnull)aPassword
                      resource:(NSString *_Nonnull)aResource
                    completion:(void (^_Nullable)(AgoraChatError *_Nullable aError))aCompletionBlock;

/**
 *  
 *  Kicks a user from your app installed on all the devices.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aUsername     The username.
 *  @param aPassword     The password.
 *
 *  @result  Returns nil on success, and the description of the issue that cause the call to fail.
 */
- (AgoraChatError *_Nullable)kickAllDevicesWithUsername:(NSString *_Nonnull)aUsername
                                        password:(NSString *_Nonnull)aPassword;

/**
 *  
 *  Kicks a single user from your app installed on all the devices.
 *
 *  This is an asynchronous method.
 *
 *  @param aUsername      The username.
 *  @param aPassword      The password.
 *  @param aCompletionBlock The completion block, which contains the error message if the method fails.
 */
- (void)kickAllDevicesWithUsername:(NSString *_Nonnull)aUsername
                          password:(NSString *_Nonnull)aPassword
                        completion:(void (^_Nullable)(AgoraChatError *_Nullable aError))aCompletionBlock;

#pragma mark - iOS

/**
 *  
 *  Migrate the chat database to the latest SDK version.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @result   Returns YES for success and the description of the cause if the method fails.
 */
- (BOOL)migrateDatabaseToLatestSDK;

/**
 *  
 *  Disconnects from the chat server when the app is switched to background.
 *
 *  @param aApplication  UIApplication
 */
- (void)applicationDidEnterBackground:(id _Nonnull )aApplication;

/**
 *  
 *  Reconnect to the server when your app returns to foreground mode.
 *
 *  @param aApplication  The current application instance.
 */
- (void)applicationWillEnterForeground:(id _Nonnull )aApplication;

/**
 *  
 *  Occurs when your app is running in the foreground and the device receives an Apple Push Notification (APN).
 *
 *  @param application  The current application instance.
 *  @param userInfo     The push content.
 */
- (void)application:(id _Nonnull)application didReceiveRemoteNotification:(NSDictionary * _Nullable)userInfo;

#pragma mark - Service Check

/**
 *  
 *  Run the server diagnostic tests for Agora Chat for a specific user.
 *  These tests are run in the order defined by AgoraChatServerCheckType.
 *  If the user is logged in, the login account is used by default.
 *
 *  This is an asynchronous method.
 *
 *  @param aUsername    The username.
 *  @param aPassword    The password.
 *  @param aCompletionBlock The completion block, which contains the error message if the method fails.
 */
- (void)serviceCheckWithUsername:(NSString *_Nonnull)aUsername
                        password:(NSString *_Nonnull)aPassword
                      completion:(void (^_Nullable)(AgoraChatServerCheckType aType, AgoraChatError *_Nullable aError))aCompletionBlock;

@property (nonatomic, strong, readonly) id<IAgoraChatTranslateManager> _Nonnull translateManager EM_DEPRECATED_IOS(3_8_9, 3_9_5,"Use -IAgoraChatManager translateMessage: instead");
@end
