/**
 *  
 *  @header IAgoraChatManager.h
 *  @abstract This protocol defines the operations of chat.
 *  @author Hyphenate
 *  @version 3.00
 */

#import <Foundation/Foundation.h>

#import "AgoraChatCommonDefs.h"
#import "AgoraChatManagerDelegate.h"
#import "AgoraChatConversation.h"

#import "AgoraChatMessage.h"
#import "AgoraChatTextMessageBody.h"
#import "AgoraChatLocationMessageBody.h"
#import "AgoraChatCmdMessageBody.h"
#import "AgoraChatFileMessageBody.h"
#import "AgoraChatImageMessageBody.h"
#import "AgoraChatVoiceMessageBody.h"
#import "AgoraChatVideoMessageBody.h"
#import "AgoraChatCustomMessageBody.h"
#import "AgoraChatCursorResult.h"

#import "AgoraChatGroupMessageAck.h"
#import "AgoraChatTranslateLanguage.h"

/**
 *  
 *  The message search direction type.
 */
typedef NS_ENUM(NSUInteger, AgoraChatMessageFetchHistoryDirection) {
    AgoraChatMessageFetchHistoryDirectionUp  = 0,    /**  The  older messages type.*/
    AgoraChatMessageFetchHistoryDirectionDown        /**  The  newer messages type.*/
};


@class AgoraChatError;

/**
 *  
 *  This protocol defines the operations of chat.
 *  The current messages are loaded from the local database, not from the server.
 */
@protocol IAgoraChatManager <NSObject>

@required

#pragma mark - Delegate

/**
 *  
 *  Adds a delegate.
 *
 *  @param aDelegate  The object that implements the protocol.
 *  @param aQueue     (optional) The queue of calling delegate methods. If you want to run the app on the main thread, set this parameter as nil.
 */
- (void)addDelegate:(id<AgoraChatManagerDelegate> _Nullable)aDelegate
      delegateQueue:(dispatch_queue_t _Nullable)aQueue;

/**
 *  
 *  Removes a delegate.
 *
 *  @param aDelegate  The delegate to be removed.
 */
- (void)removeDelegate:(id<AgoraChatManagerDelegate> _Nonnull)aDelegate;

#pragma mark - Conversation

/**
 *  
 *  Gets all conversations in the local database. The SDK loads the conversations from the cache first. If no conversation is in the cache, the SDK loads from the local database.
 *
 *  @result The conversation NSArray.
 */
- (NSArray<AgoraChatConversation *> * _Nullable)getAllConversations;

/**
 *  
 *  Gets all conversations from the server.
 * 
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 */
- (void)getConversationsFromServer:(void (^_Nullable)(NSArray<AgoraChatConversation *> * _Nullable aConversations, AgoraChatError * _Nullable aError))aCompletionBlock;

/**
 *  
 *  Gets a conversation from the local database.
 *
 *  @param aConversationId  The conversation ID.
 *
 *  @result The conversation object.
 */
- (AgoraChatConversation *_Nullable)getConversationWithConvId:(NSString * _Nullable)aConversationId;

/**
 *  
 *  Gets a conversation from the local database.
 *
 *  @param aConversationId  The conversation ID.
 *  @param aType            The conversation type (Must be specified).
 *  @param aIfCreate        Whether to create the conversation if it does not exist.
 *
 *  @result The conversation.
 */
- (AgoraChatConversation *_Nullable)getConversation:(NSString *_Nonnull)aConversationId
                               type:(AgoraChatConversationType)aType
                   createIfNotExist:(BOOL)aIfCreate;

/**
 *  
 *  Gets a conversation from the local database.
 *
 *  @param aConversationId  The conversation ID.
 *  @param aType            The conversation type (Must be specified).
 *  @param aIfCreate        Whether to create the conversation if it does not exist.
 *  @param isThread  Whether it is a threadChat type of session
 *  @result The conversation.
 */
- (AgoraChatConversation *_Nullable)getConversation:(NSString *_Nonnull)aConversationId
                               type:(AgoraChatConversationType)aType
                   createIfNotExist:(BOOL)aIfCreate isThread:(BOOL)isThread;

/**
 *  
 *  Deletes a conversation from the local database.
 *  @param aConversationId      The conversation ID.
 *  @param aIsDeleteMessages    Whether to delete the messages in the conversation.
 *  - `Yes`: Yes;
 *  - `No`: No.
 *  @param aCompletionBlock     The completion block, which contains the error message if the method call fails.
 *
 */
- (void)deleteConversation:(NSString * _Nonnull)aConversationId
          isDeleteMessages:(BOOL)aIsDeleteMessages
                completion:(void (^_Nullable)(NSString * _Nullable aConversationId, AgoraChatError *_Nullable aError))aCompletionBlock;

/*!
  *  
  *  Deletes a conversation from the server.
  *
  *  @param aConversationId      The conversation ID.
  *  @param aConversationType    The conversation type.
  *  @param aIsDeleteMessages    Whether to delete the related messages with the conversation.
  *                          - `Yes`: Yes;
  *                          - `No`: No.
  *  @param aCompletionBlock     The completion block, which contains the error message if the method call fails.
  *
  */
 - (void)deleteServerConversation:(NSString * _Nonnull)aConversationId
                 conversationType:(AgoraChatConversationType)aConversationType
           isDeleteServerMessages:(BOOL)aIsDeleteServerMessages
                       completion:(void (^_Nullable)(NSString * _Nullable aConversationId, AgoraChatError * _Nullable aError))aCompletionBlock;

/*!
 *  
 *  Deletes multiple conversations.
 *
 *  @param aConversations       The conversation list.
 *  @param aIsDeleteMessages    Whether to delete the messages.
 *  - `Yes`: Yes;
 *  - `No`: No.
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 *
 */
- (void)deleteConversations:(NSArray<AgoraChatConversation *> * _Nullable)aConversations
           isDeleteMessages:(BOOL)aIsDeleteMessages
                 completion:(void (^_Nullable)(AgoraChatError * _Nullable aError))aCompletionBlock;

/**
 *  
 *  Imports multiple conversations to the local database.
 *
 *  @param aConversations       The conversation list.
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 *
 */
- (void)importConversations:(NSArray<AgoraChatConversation *> * _Nullable)aConversations
                 completion:(void (^ _Nullable)(AgoraChatError * _Nullable aError))aCompletionBlock;

#pragma mark - Message

/**
 *  
 *  Gets the specified message.
 *
 *  @param aMessageId    The message ID.
 *  @result AgoraChatMessage     The message content.
 */
- (AgoraChatMessage * _Nullable)getMessageWithMessageId:(NSString * _Nonnull)aMessageId;

/**
 *  
 *  Gets the local path of message attachments in a conversation. Delete the conversation will also delete the files under the file path.
 *
 *  @param aConversationId  The conversation ID.
 *
 *  @result The attachment path.
 */
- (NSString * _Nullable)getMessageAttachmentPath:(NSString * _Nonnull)aConversationId;

/**
 *  
 *  Imports multiple messages to the local database.
 *
 *  @param aMessages            The message NSArray.
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 *
 */
- (void)importMessages:(NSArray<AgoraChatMessage *> * _Nonnull)aMessages
            completion:(void (^_Nullable)(AgoraChatError *_Nullable aError))aCompletionBlock;

/**
 *  
 *  Updates a message in the local database. Latest Message of the conversation and other properties will be updated accordingly. MessageId of the message cannot be updated.
 *
 *  @param aMessage             The message instance.
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 *
 */
- (void)updateMessage:(AgoraChatMessage *_Nonnull)aMessage
           completion:(void (^_Nullable)(AgoraChatMessage * _Nullable aMessage, AgoraChatError * _Nullable aError))aCompletionBlock;

/**
 *  
 *  Sends the read receipt for a message.
 * 
 *  This is an asynchronous method.
 *
 *  @param aMessageId           The message ID.
 *  @param aUsername            The receiver of the read receipt.
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 *
 */
- (void)sendMessageReadAck:(NSString * _Nonnull)aMessageId
                    toUser:(NSString * _Nonnull)aUsername
                completion:(void (^_Nullable)(AgoraChatError *_Nullable aError))aCompletionBlock;


/**
 *  
 *  Sends the read receipt for a group message.
 * 
 *  This is an asynchronous method.
 *
 *  @param aMessageId           The message ID.
 *  @param aGroupId             The group receiver ID.
 *  @param aContent             The message content.
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 *
 */
- (void)sendGroupMessageReadAck:(NSString * _Nonnull)aMessageId
                        toGroup:(NSString * _Nonnull)aGroupId
                        content:(NSString * _Nullable)aContent
                     completion:(void (^_Nullable)(AgoraChatError *_Nullable aError))aCompletionBlock;

/**
 *  
 *  Sends the conversation read receipt to the server.
 * 
 *  This method applies to one-to-one chats only.
 * 
 *  This method call notifies the server to set the number of unread messages of the specified conversation as 0, and triggers the onConversationRead callback on the receiver's client.
 *
 *  To reduce the number of method calls, we recommend that you call this method when the user enters a conversation with many unread messages, and call sendMessageReadAck during a conversation to send the message read receipts.
 * 
 *  This is an asynchronous method.
 * 
 *  @param conversationId          The conversation ID.
 *  @param aCompletionBlock        The completion block, which contains the error message if the method fails.
 * 
 */
- (void)ackConversationRead:(NSString * _Nonnull)conversationId
                 completion:(void (^_Nullable)(AgoraChatError *_Nullable aError))aCompletionBlock;

/**
 *  
 *  Recalls a message.
 *
 *  This is an asynchronous method.
 *
 *  @param aMessageId           The message ID
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 *
 */
- (void)recallMessageWithMessageId:(NSString *_Nonnull)aMessageId
                        completion:(void (^_Nullable)(AgoraChatError *_Nullable aError))aCompletionBlock;


/**
 *  
 *  Sends a message.
 * 
 *  This is an asynchronous method.
 *
 *  @param aMessage             The message instance.
 *  @param aProgressBlock       The block of attachment upload progress in percentage, 0~100.
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 */
- (void)sendMessage:(AgoraChatMessage *_Nonnull)aMessage
           progress:(void (^_Nullable)(int progress))aProgressBlock
         completion:(void (^_Nullable)(AgoraChatMessage *_Nullable message, AgoraChatError *_Nullable error))aCompletionBlock;

/**
 *  
 *  Resends a message.
 *
 *  @param aMessage             The message object.
 *  @param aProgressBlock       The callback block of attachment upload progress.
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 */
- (void)resendMessage:(AgoraChatMessage *_Nonnull)aMessage
             progress:(void (^_Nullable)(int progress))aProgressBlock
           completion:(void (^_Nullable)(AgoraChatMessage *_Nullable message, AgoraChatError *_Nullable error))aCompletionBlock;

/**
 *  
 *  Downloads the message thumbnail (the thumbnail of an image message or the first frame of a video message).
 * 
 *  The SDK automatically downloads the thumbnail. If the auto-download fails, you can call this method to manually download the thumbnail.
 *
 *  @param aMessage             The message object.
 *  @param aProgressBlock       The callback block of attachment download progress.
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 */
- (void)downloadMessageThumbnail:(AgoraChatMessage *_Nonnull)aMessage
                        progress:(void (^_Nullable)(int progress))aProgressBlock
                      completion:(void (^_Nullable)(AgoraChatMessage *_Nullable message, AgoraChatError *_Nullable error))aCompletionBlock;

/**
 *  
 *  Downloads message attachment (voice, video, image or file). 
 *  
 *  SDK handles attachment downloading automatically. If automatic download failed, you can download attachment manually.
 *
 *  This is an asynchronous method.
 * 
 *  @param aMessage             The message object.
 *  @param aProgressBlock       The callback block of attachment download progress.
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 */
- (void)downloadMessageAttachment:(AgoraChatMessage *_Nonnull)aMessage
                         progress:(void (^_Nullable)(int progress))aProgressBlock
                       completion:(void (^_Nullable)(AgoraChatMessage *_Nullable message, AgoraChatError *_Nullable error))aCompletionBlock;



/**
 *  
 *  Fetches conversation messages from server.
 
 *  @param aConversationId      The conversation id which select to fetch message.
 *  @param aConversationType    The conversation type which select to fetch message.
 *  @param aStartMessageId      The start message id, if empty, start from the server‘s last message.
 *  @param direction      AgoraChatMessageFetchHistoryDirection up or down
 *  @param aPageSize            The page size.
 *  @param pError               The error information if the method fails: Error.
 *
 *  @result    The messages list.
 */
- (AgoraChatCursorResult<AgoraChatMessage*> *_Nullable)fetchHistoryMessagesFromServer:(NSString *_Nonnull)aConversationId
                                  conversationType:(AgoraChatConversationType)aConversationType
                                    startMessageId:(NSString *_Nullable)aStartMessageId
                                       fetchDirection:(AgoraChatMessageFetchHistoryDirection)direction
                                          pageSize:(int)aPageSize
                                             error:(AgoraChatError **_Nullable)pError;
/**
 *  
 *  Fetches conversation messages from server.
 
 *  @param aConversationId      The conversation id which select to fetch message.
 *  @param aConversationType    The conversation type which select to fetch message.
 *  @param aStartMessageId      The start message id, if empty, start from the server‘s last message.
 *  @param aPageSize            The page size.
 *  @param pError               The error information if the method fails: Error.
 *
 *  @result    The messages list.
 */
- (AgoraChatCursorResult<AgoraChatMessage*> *_Nullable)fetchHistoryMessagesFromServer:(NSString *_Nonnull)aConversationId
                                  conversationType:(AgoraChatConversationType)aConversationType
                                    startMessageId:(NSString *_Nullable)aStartMessageId
                                          pageSize:(int)aPageSize
                                             error:(AgoraChatError **_Nullable)pError;


/**
 *  
 *  Fetches conversation messages from server.
 * 
 *  This is an asynchronous method.
 * 
 *  @param aConversationId      The conversation id which select to fetch message.
 *  @param aConversationType    The conversation type which select to fetch message.
 *  @param aStartMessageId      The start message id, if empty, start from the server last message.
 *  @param aPageSize            The page size.
 *  @param aCompletionBlock     The callback block of fetch complete, which contains the error message if the method fails.
 */
- (void)asyncFetchHistoryMessagesFromServer:(NSString *_Nonnull)aConversationId
                           conversationType:(AgoraChatConversationType)aConversationType
                             startMessageId:(NSString *_Nullable)aStartMessageId
                                   pageSize:(int)aPageSize
                                 completion:(void (^_Nullable)(AgoraChatCursorResult<AgoraChatMessage*> *_Nullable aResult, AgoraChatError *_Nullable aError))aCompletionBlock;
/**
 *  
 *  Fetches conversation messages from server.
 *
 *  This is an asynchronous method.
 *
 *  @param aConversationId      The conversation id which select to fetch message.
 *  @param aConversationType    The conversation type which select to fetch message.
 *  @param aStartMessageId      The start message id, if empty, start from the server last message.
 *  @param direction      AgoraChatMessageFetchHistoryDirection up or down
 *  @param aPageSize            The page size. limit 50
 *  @param aCompletionBlock     The callback block of fetch complete, which contains the error message if the method fails.
 */
- (void)asyncFetchHistoryMessagesFromServer:(NSString *_Nonnull)aConversationId
                           conversationType:(AgoraChatConversationType)aConversationType
                             startMessageId:(NSString *_Nullable)aStartMessageId
                             fetchDirection:(AgoraChatMessageFetchHistoryDirection)direction
                                   pageSize:(int)aPageSize
                                 completion:(void (^_Nullable)(AgoraChatCursorResult<AgoraChatMessage*> *_Nullable aResult, AgoraChatError *_Nullable aError))aCompletionBlock;

 


/**
 *  
 *  Fetches the read back receipt of specified group messages from the server.
 * 
 *  This is an asynchronous method.
 * 
 *  @param  aMessageId           The message ID which select to fetch.
 *  @param  aGroupId             The group ID which select to fetch.
 *  @param  aGroupAckId          The group ack ID which select to fetch.
 *  @param  aPageSize            The page size.
 *  @param  aCompletionBlock     The callback block of fetch complete, which contains the error message if the method fails.
 */
- (void)asyncFetchGroupMessageAcksFromServer:(NSString *_Nonnull)aMessageId
                                     groupId:(NSString *_Nonnull)aGroupId
                             startGroupAckId:(NSString *_Nonnull)aGroupAckId
                                    pageSize:(int)aPageSize
                                  completion:(void (^_Nullable)(AgoraChatCursorResult<AgoraChatGroupMessageAck *> *_Nullable aResult, AgoraChatError *_Nullable error, int totalCount))aCompletionBlock;

/**
 *  
 *  Report violation message
 
 *  @param  aMessageId              Violation Message ID
 *  @param  aTag                            Report tag
 *  @param  aReason                     Report reasons
 *  @param  aCompletion            The callback block of fetch complete
 */
- (void)reportMessageWithId:(NSString *_Nonnull )aMessageId
                        tag:(NSString *_Nonnull)aTag
                     reason:(NSString *_Nonnull)aReason
                 completion:(void(^_Nullable)(AgoraChatError* _Nullable error))aCompletion;

/*!
 *  
 *  Deletes messages with timestamp that is before the specified one.
 *
 *  @param aTimestamp            The specified Unix timestamp(miliseconds).
 *  @param aCompletion           The completion block, which contains the error message if the method fails.
 *
 */
- (void)deleteMessagesBefore:(NSUInteger)aTimestamp
                  completion:(void(^)(AgoraChatError*error))aCompletion;

/*!
 *  
 *  Translate a message.
 *
 *  @param aMessage             The message object.
 *  @param aLanguages           The target languages.
 *  @param aCompletionBlock     The completion block, which contains the error message if the method fails.
 */
- (void)translateMessage:(AgoraChatMessage *)aMessage
         targetLanguages:(NSArray<NSString*>*)aLanguages
              completion:(void (^)(AgoraChatMessage *message, AgoraChatError *error))aCompletionBlock;

/*!
 *  
 *
 *  Gets the list of supported languages for translation.
 *
 *  @param aCompletionBlock The completion block, which contains the error message if the method fails.
 */
- (void)fetchSupportedLanguages:(void(^_Nullable)(NSArray<AgoraChatTranslateLanguage*>* _Nullable languages,AgoraChatError* _Nullable error))aCompletionBlock;

/**
 *  
 *  Loads messages with the specified keyword from the local database.
 * 
 *  This method returns messages in the sequence of the timestamp when they are received.
 * 
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aType            The message type to load.
 *  @param aTimestamp       The reference timestamp for the messages to be loaded. If you set this parameter as a negative value, the SDK loads messages from the latest.
 *  @param aCount           The number of messages to load. If you set this parameter less than 1, it will be handled as count=1.
 *  @param aUsername        Message sender (optional). Use aUsername=nil to ignore.
 *  @param aDirection       The message search directions: {@link MessageSearchDirection}.
 *                          - `UP`: In the reverse chronological order.
 *                          - `DOWN`：In the chronological order.
 *
 *  @result AgoraChatMessage NSArray.
 *
 */
- (NSArray<AgoraChatMessage *> *)loadMessagesWithType:(AgoraChatMessageBodyType)aType
                   timestamp:(long long)aTimestamp
                       count:(int)aCount
                    fromUser:(NSString*)aUsername
             searchDirection:(AgoraChatMessageSearchDirection)aDirection;

/**
 *  
 *  Loads messages with the specified keyword from the local database.
 * 
 *  This method returns messages in the sequence of the timestamp when they are received.
 *
 *  @param aType            The message type to load.
 *  @param aTimestamp       The reference timestamp for the messages to be loaded. If you set this parameter as a negative value, the SDK loads messages from the latest.
 *  @param aCount           The number of messages to load. If you set this parameter less than 1, it will be handled as count=1
 *  @param aUsername        The user that sends the message. Setting it as nil means that the SDK ignores this parameter.
 *  @param aDirection       The message search directions: {@link MessageSearchDirection}.
 *                          - `UP`: In the reverse chronological order.
 *                          - `DOWN`：In the chronological order.
 *  @param aCompletionBlock The completion block, which contains the error message if the method fails.
 *
 */
- (void)loadMessagesWithType:(AgoraChatMessageBodyType)aType
                   timestamp:(long long)aTimestamp
                       count:(int)aCount
                    fromUser:(NSString*)aUsername
             searchDirection:(AgoraChatMessageSearchDirection)aDirection
                  completion:(void (^)(NSArray<AgoraChatMessage *> *aMessages, AgoraChatError *aError))aCompletionBlock;

/**
 *  
 *  Loads messages with the specified keyword from the local database.
 *
 *  This method returns messages in the sequence of the timestamp when they are received.
 *
 *  This is a synchronous method and blocks the current thread.
 *
 *  @param aKeyword         The keyword for searching the messages. Setting it as nil means that the SDK ignores this parameter.
 *  @param aTimestamp       The reference timestamp for the messages to be loaded. If you set this parameter as a negative value, the SDK loads messages from the latest.
 *  @param aCount           The number of messages to load. If you set this parameter less than 1, it will be handled as count=1.
 *  @param aSender          The user that sends the message.
 *  @param aDirection       The message search directions: {@link MessageSearchDirection}.
 *                          - `UP`: In the reverse chronological order.
 *                          - `DOWN`：In the chronological order.
 *
 *  @result AgoraChatMessage NSArray. <AgoraChatMessage>
 *
 */
- (NSArray<AgoraChatMessage *> *)loadMessagesWithKeyword:(NSString*)aKeywords
                      timestamp:(long long)aTimestamp
                          count:(int)aCount
                       fromUser:(NSString*)aSender
                searchDirection:(AgoraChatMessageSearchDirection)aDirection;

/**
 *  
 *  Loads messages with the specified keyword from the local database.
 * 
 *  This method returns messages in the sequence of the timestamp when they are received.
 *
 *  @param aKeyword         The keyword for searching the messages. Setting it as nil means that the SDK ignores this parameter.
 *  @param aTimestamp       The reference timestamp for the messages to be loaded. If you set this parameter as a negative value, the SDK loads messages from the latest.
 *  @param aCount           The number of messages to load. If you set this parameter less than 1, it will be handled as count=1.
 AgoraChatMessageSearchDirectionDown: get aCount of messages after aMessageId *  ----
 *  @param aSender          The sender of the message. Setting it as nil means that the SDK ignores this parameter.
 *  @param aDirection       The message search directions: {@link MessageSearchDirection}.
 *                          - `UP`: In the reverse chronological order.
 *                          - `DOWN`：In the chronological order.
 *  @param aCompletionBlock  The completion block which contains the error code and error information if the method fails.
 */
- (void)loadMessagesWithKeyword:(NSString*)aKeywords
                      timestamp:(long long)aTimestamp
                          count:(int)aCount
                       fromUser:(NSString*)aSender
                searchDirection:(AgoraChatMessageSearchDirection)aDirection
                     completion:(void (^)(NSArray<AgoraChatMessage *> *aMessages, AgoraChatError *aError))aCompletionBlock;

NS_ASSUME_NONNULL_BEGIN
/*!
 *  
 *  Adds a Reaction.
 *
 *  @param reaction  The Reaction content.
 *  @param messageId  The message ID.
 *  @param completion  The completion block which contains the error code and error information if the method fails.
 */
- (void)addReaction:(NSString *)reaction toMessage:(NSString *)messageId completion:(nullable void(^)(AgoraChatError * _Nullable))completion;

/*!
 *  
 *  Removes a Reaction.
 *
 *  @param reaction  The Reaction content.
 *  @param messageId  The message ID.
 *  @param completion  The completion block which contains the error code and error information if the method fails.
 */
- (void)removeReaction:(NSString *)reaction fromMessage:(NSString *)messageId completion:(nullable void(^)(AgoraChatError * _Nullable))completion;

/*!
 *  
 *  Gets the Reaction list.
 *
 *  @param messageId  The message ID.
 *  @param groupId  The group ID. This parameter is invalid only for group chat.
 *  @param chatType  The chat type. Only one-to-one chat ({@link AgoraChatTypeChat} and group chat ({@link AgoraChatTypeGroupChat}) are allowed.
 *  @param completion The completion block which contains the error code and error information if the method fails.
 */
- (void)getReactionList:(NSArray <NSString *>*)messageIds
                groupId:(nullable NSString *)groupId
               chatType:(AgoraChatType)chatType
             completion:(void (^)(NSDictionary <NSString *, NSArray<AgoraChatMessageReaction *> *> *, AgoraChatError * _Nullable))completion;

/*!
 *  
 *  Gets the Reaction detail list of a chat group message with pagination.
 *
 *  @param messageId  The message ID.
 *  @param reaction  The Reaction content.
 *  @param cursor  The cursor that specifies where to start to get data. If it is `nil` or `@""`, the SDK retrieves Reactions in the chronological order of their creation time.
 *  @param pageSize   The number of Reactions that you expect to get on each page. The value cannot exceed 100.
 *  @param completion  The completion block, which contains the Reaction list and the cursor for the next query. When the cursor is `nil`, all data is fetched.
 */
- (void)getReactionDetail:(NSString *)messageId
                 reaction:(NSString *)reaction
                    cursor:(nullable NSString *)cursor
                 pageSize:(uint64_t)pageSize
               completion:(void (^)(AgoraChatMessageReaction *, NSString * _Nullable cursor, AgoraChatError * _Nullable))completion;

NS_ASSUME_NONNULL_END

@end
